/***************************************************

 These clauses define a sample proximity relation.
 
 Relation R2, 2023-03-21.
 Last version: 2023-04-30

***************************************************/

:-include('generic.pl').

% A proximity equation between two nonconstant terms.
% Both symmetric versions should be written,
% unless the symmetric version causes looping.

prox(f(X), g(Y), 0.7) :-
  % Restricting constraint  
    prox(X, a, Alpha),
    (Alpha >= 0.5),
    prox(Y, d, Beta),
    (Beta >= 0.8).
prox(g(Y), f(X), 0.7) :-
    prox(f(X), g(Y), 0.7).

% Proximity equations between different constants.
% Both symmetric versions should be written.

prox(a, b, 0.5).
prox(b, a, 0.5).

prox(a, c, 0.7).
prox(c, a, 0.7).

prox(b, d, 0.7).
prox(d, b, 0.7).

prox(c, d, 0.8).
prox(d, c, 0.8).




/*****************************************

Sample goals:

?- solve(g(Y), f(X), 0.4, Deg). 
Y = c,
X = b,
Deg = 0.7 ;
Y = d,
X = b,
Deg = 0.7 ;
Y = X, X = c,
Deg = 0.7 ;
Y = d,
X = c,
Deg = 0.7 ;
Y = c,
X = a,
Deg = 0.7 ;
Y = d,
X = a,
Deg = 0.7 ;
false.

?- solve(g(Y), f(X), 0.6, Deg). 
Y = c,
X = b,
Deg = 0.7 ;
Y = d,
X = b,
Deg = 0.7 ;
Y = X, X = c,
Deg = 0.7 ;
Y = d,
X = c,
Deg = 0.7 ;
Y = c,
X = a,
Deg = 0.7 ;
Y = d,
X = a,
Deg = 0.7 ;
false.




*****************************************/

% Unit tests.
% To test, evaluate run_tests(rel2).

:- begin_tests(rel2).

test(solve1) :-
    set_prolog_flag(answer_write_options,[max_depth(0)]),
    findall([Y,X,Deg], solve(g(Y), f(X), 0.4, Deg), L),
    L == [[d,a,0.7],[c,a,0.7],[d,b,0.7],[c,b,0.7],[d,c,0.7],[c,c,0.7]].

test(solve2) :-
    findall([Y,X,Deg], solve(g(Y), f(X), 0.6, Deg), L),
    L == [[d,a,0.7],[c,a,0.7],[d,b,0.7],[c,b,0.7],[d,c,0.7],[c,c,0.7]].

test(all_instances1) :-
    all_instances(g(_), f(_), 0.4, I),
    I == [[g(c),f(a),0.7],[g(c),f(b),0.7],[g(c),f(c),0.7],[g(d),f(a),0.7],[g(d),f(b),0.7],[g(d),f(c),0.7]].

test(best_instances1) :-
    best_instances(g(_), f(_), 0.4, I),
    I == [[g(d),f(a),0.7],[g(c),f(a),0.7],[g(d),f(b),0.7],[g(c),f(b),0.7],[g(d),f(c),0.7],[g(c),f(c),0.7]].

:- end_tests(rel2).
