/***************************************************

 These clauses define a sample proximity relation.
 
 Relation R4, 2023-03-23.
 Last version: 2023-04-30

***************************************************/

:-include('generic.pl').

% Proximity equations between two nonconstant terms.
% Both symmetric versions should be written.
% unless the symmetric version causes looping.

prox(f(X), g(Y), Delta) :-
  % defining constraint   
    prox(X, Y, Delta1),
    min_list([0.7,Delta1], Delta).
prox(g(Y), f(X), Delta) :-
    prox(f(X), g(Y), Delta).

prox(p(X1,X2), q(Y1,Y2,Y3), Delta) :-
  % defining constraint   
    prox(X1, h(Y1,Y2), Delta1),
    prox(X2, f(Y3), Delta2),
    min_list([0.8,Delta1,Delta2], Delta).
prox(q(Y1,Y2,Y3), p(X1,X2), Delta) :-
    prox(p(X1,X2), q(Y1,Y2,Y3), Delta).


% Proximity equations between different constants.
% Both symmetric versions should be written.

prox(a, b, 0.5).
prox(b, a, 0.5).

prox(a, c, 0.7).
prox(c, a, 0.7).

prox(b, d, 0.7).
prox(d, b, 0.7).

prox(c, d, 0.8).
prox(d, c, 0.8).




/*****************************************

Sample goals:

?- solve(p(X1,X2), q(a,d,a), 0.5, Deg).
X1 = h(a,d),
X2 = f(a),
Deg = 0.8 ;
X1 = h(a,d),
X2 = f(b),
Deg = 0.5 ;
X1 = h(a,d),
X2 = f(c),
Deg = 0.7 ;
X1 = h(a,d),
X2 = g(a),
Deg = 0.7 ;
X1 = h(a,d),
X2 = g(b),
Deg = 0.5 ;
X1 = h(a,d),
X2 = g(c),
Deg = 0.7 ;
X1 = h(a,b),
X2 = f(a),
Deg = 0.7 ;
X1 = h(a,b),
X2 = f(b),
Deg = 0.5 ;
X1 = h(a,b),
X2 = f(c),
Deg = 0.7 ;
X1 = h(a,b),
X2 = g(a),
Deg = 0.7 ;
X1 = h(a,b),
X2 = g(b),
Deg = 0.5 ;
X1 = h(a,b),
X2 = g(c),
Deg = 0.7 ;
X1 = h(a,c),
X2 = f(a),
Deg = 0.8 ;
X1 = h(a,c),
X2 = f(b),
Deg = 0.5 ;
X1 = h(a,c),
X2 = f(c),
Deg = 0.7 ;
X1 = h(a,c),
X2 = g(a),
Deg = 0.7 ;
X1 = h(a,c),
X2 = g(b),
Deg = 0.5 ;
X1 = h(a,c),
X2 = g(c),
Deg = 0.7 ;
X1 = h(b,d),
X2 = f(a),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(b),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(c),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(a),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(b),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(c),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(a),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(b),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(c),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(a),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(b),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(c),
Deg = 0.5 ;
X1 = h(b,c),
X2 = f(a),
Deg = 0.5 ;
X1 = h(b,c),
X2 = f(b),
Deg = 0.5 ;
X1 = h(b,c),
X2 = f(c),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(a),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(b),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(c),
Deg = 0.5 ;
X1 = h(c,d),
X2 = f(a),
Deg = 0.7 ;
X1 = h(c,d),
X2 = f(b),
Deg = 0.5 ;
X1 = h(c,d),
X2 = f(c),
Deg = 0.7 ;
X1 = h(c,d),
X2 = g(a),
Deg = 0.7 ;
X1 = h(c,d),
X2 = g(b),
Deg = 0.5 ;
X1 = h(c,d),
X2 = g(c),
Deg = 0.7 ;
X1 = h(c,b),
X2 = f(a),
Deg = 0.7 ;
X1 = h(c,b),
X2 = f(b),
Deg = 0.5 ;
X1 = h(c,b),
X2 = f(c),
Deg = 0.7 ;
X1 = h(c,b),
X2 = g(a),
Deg = 0.7 ;
X1 = h(c,b),
X2 = g(b),
Deg = 0.5 ;
X1 = h(c,b),
X2 = g(c),
Deg = 0.7 ;
X1 = h(c,c),
X2 = f(a),
Deg = 0.7 ;
X1 = h(c,c),
X2 = f(b),
Deg = 0.5 ;
X1 = h(c,c),
X2 = f(c),
Deg = 0.7 ;
X1 = h(c,c),
X2 = g(a),
Deg = 0.7 ;
X1 = h(c,c),
X2 = g(b),
Deg = 0.5 ;
X1 = h(c,c),
X2 = g(c),
Deg = 0.7 ;
false.
.

% 54 solutions

?- solve(p(X1,X2), q(a,d,f(a)), 0.5, Deg).
X1 = h(a,d),
X2 = f(f(a)),
Deg = 0.8 ;
X1 = h(a,d),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(a,d),
X2 = f(f(c)),
Deg = 0.7 ;
X1 = h(a,d),
X2 = f(g(a)),
Deg = 0.7 ;
X1 = h(a,d),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(a,d),
X2 = f(g(c)),
Deg = 0.7 ;
X1 = h(a,d),
X2 = g(f(a)),
Deg = 0.7 ;
X1 = h(a,d),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(a,d),
X2 = g(f(c)),
Deg = 0.7 ;
X1 = h(a,d),
X2 = g(g(a)),
Deg = 0.7 ;
X1 = h(a,d),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(a,d),
X2 = g(g(c)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = f(f(a)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(a,b),
X2 = f(f(c)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = f(g(a)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(a,b),
X2 = f(g(c)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = g(f(a)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(a,b),
X2 = g(f(c)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = g(g(a)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(a,b),
X2 = g(g(c)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = f(f(a)),
Deg = 0.8 ;
X1 = h(a,c),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(a,c),
X2 = f(f(c)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = f(g(a)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(a,c),
X2 = f(g(c)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = g(f(a)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(a,c),
X2 = g(f(c)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = g(g(a)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(a,c),
X2 = g(g(c)),
Deg = 0.7 ;
X1 = h(b,d),
X2 = f(f(a)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(f(c)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(g(a)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(g(c)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(f(a)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(f(c)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(g(a)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(g(c)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(f(a)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(f(c)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(g(a)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(g(c)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(f(a)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(f(c)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(g(a)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(g(c)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = f(f(a)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = f(f(c)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = f(g(a)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = f(g(c)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(f(a)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(f(c)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(g(a)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(g(c)),
Deg = 0.5 ;
X1 = h(c,d),
X2 = f(f(a)),
Deg = 0.7 ;
X1 = h(c,d),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(c,d),
X2 = f(f(c)),
Deg = 0.7 ;
X1 = h(c,d),
X2 = f(g(a)),
Deg = 0.7 ;
X1 = h(c,d),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(c,d),
X2 = f(g(c)),
Deg = 0.7 ;
X1 = h(c,d),
X2 = g(f(a)),
Deg = 0.7 ;
X1 = h(c,d),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(c,d),
X2 = g(f(c)),
Deg = 0.7 ;
X1 = h(c,d),
X2 = g(g(a)),
Deg = 0.7 ;
X1 = h(c,d),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(c,d),
X2 = g(g(c)),
Deg = 0.7 ;
X1 = h(c,b),
X2 = f(f(a)),
Deg = 0.7 ;
X1 = h(c,b),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(c,b),
X2 = f(f(c)),
Deg = 0.7 ;
X1 = h(c,b),
X2 = f(g(a)),
Deg = 0.7 ;
X1 = h(c,b),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(c,b),
X2 = f(g(c)),
Deg = 0.7 ;
X1 = h(c,b),
X2 = g(f(a)),
Deg = 0.7 ;
X1 = h(c,b),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(c,b),
X2 = g(f(c)),
Deg = 0.7 ;
X1 = h(c,b),
X2 = g(g(a)),
Deg = 0.7 ;
X1 = h(c,b),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(c,b),
X2 = g(g(c)),
Deg = 0.7 ;
X1 = h(c,c),
X2 = f(f(a)),
Deg = 0.7 ;
X1 = h(c,c),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(c,c),
X2 = f(f(c)),
Deg = 0.7 ;
X1 = h(c,c),
X2 = f(g(a)),
Deg = 0.7 ;
X1 = h(c,c),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(c,c),
X2 = f(g(c)),
Deg = 0.7 ;
X1 = h(c,c),
X2 = g(f(a)),
Deg = 0.7 ;
X1 = h(c,c),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(c,c),
X2 = g(f(c)),
Deg = 0.7 ;
X1 = h(c,c),
X2 = g(g(a)),
Deg = 0.7 ;
X1 = h(c,c),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(c,c),
X2 = g(g(c)),
Deg = 0.7 ;
false.

% 108 solutions

?- solve(p(X1,f(X2)), q(a,d,g(f(a))), 0.5, Deg).
X1 = h(a,d),
X2 = g(f(a)),
Deg = 0.8 ;
X1 = h(a,d),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(a,d),
X2 = g(f(c)),
Deg = 0.7 ;
X1 = h(a,d),
X2 = g(g(a)),
Deg = 0.7 ;
X1 = h(a,d),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(a,d),
X2 = g(g(c)),
Deg = 0.7 ;
X1 = h(a,d),
X2 = f(f(a)),
Deg = 0.7 ;
X1 = h(a,d),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(a,d),
X2 = f(f(c)),
Deg = 0.7 ;
X1 = h(a,d),
X2 = f(g(a)),
Deg = 0.7 ;
X1 = h(a,d),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(a,d),
X2 = f(g(c)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = g(f(a)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(a,b),
X2 = g(f(c)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = g(g(a)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(a,b),
X2 = g(g(c)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = f(f(a)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(a,b),
X2 = f(f(c)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = f(g(a)),
Deg = 0.7 ;
X1 = h(a,b),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(a,b),
X2 = f(g(c)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = g(f(a)),
Deg = 0.8 ;
X1 = h(a,c),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(a,c),
X2 = g(f(c)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = g(g(a)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(a,c),
X2 = g(g(c)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = f(f(a)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(a,c),
X2 = f(f(c)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = f(g(a)),
Deg = 0.7 ;
X1 = h(a,c),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(a,c),
X2 = f(g(c)),
Deg = 0.7 ;
X1 = h(b,d),
X2 = g(f(a)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(f(c)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(g(a)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = g(g(c)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(f(a)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(f(c)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(g(a)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(b,d),
X2 = f(g(c)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(f(a)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(f(c)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(g(a)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(g(b)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = g(g(c)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(f(a)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(f(b)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(f(c)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(g(a)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(g(b)),
Deg = 0.5 ;
X1 = h(b,b),
X2 = f(g(c)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(f(a)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(f(b)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(f(c)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(g(a)),
Deg = 0.5 ;
X1 = h(b,c),
X2 = g(g(b)),
Deg = 0.5 

% 108 solutions.

?- best_instances(p(X1,X2), q(a,d,f(a)), 0.5, I).
I = [[p(h(a,d),f(f(a))),0.8],[p(h(a,c),f(f(a))),0.8]].


*****************************************/

% Unit tests.
% To test, evaluate run_tests(rel4).

:- begin_tests(rel4).

test(solve1) :-
    set_prolog_flag(answer_write_options,[max_depth(0)]),
    findall([X1,X2,Deg], solve(p(X1,X2), q(a,d,a), 0.5, Deg), L),
    L == [[h(a,d),f(a),0.8],[h(a,d),f(b),0.5],[h(a,d),f(c),0.7],[h(a,d),g(a),0.7],[h(a,d),g(b),0.5],[h(a,d),g(c),0.7],[h(a,b),f(a),0.7],[h(a,b),f(b),0.5],[h(a,b),f(c),0.7],[h(a,b),g(a),0.7],[h(a,b),g(b),0.5],[h(a,b),g(c),0.7],[h(a,c),f(a),0.8],[h(a,c),f(b),0.5],[h(a,c),f(c),0.7],[h(a,c),g(a),0.7],[h(a,c),g(b),0.5],[h(a,c),g(c),0.7],[h(b,d),f(a),0.5],[h(b,d),f(b),0.5],[h(b,d),f(c),0.5],[h(b,d),g(a),0.5],[h(b,d),g(b),0.5],[h(b,d),g(c),0.5],[h(b,b),f(a),0.5],[h(b,b),f(b),0.5],[h(b,b),f(c),0.5],[h(b,b),g(a),0.5],[h(b,b),g(b),0.5],[h(b,b),g(c),0.5],[h(b,c),f(a),0.5],[h(b,c),f(b),0.5],[h(b,c),f(c),0.5],[h(b,c),g(a),0.5],[h(b,c),g(b),0.5],[h(b,c),g(c),0.5],[h(c,d),f(a),0.7],[h(c,d),f(b),0.5],[h(c,d),f(c),0.7],[h(c,d),g(a),0.7],[h(c,d),g(b),0.5],[h(c,d),g(c),0.7],[h(c,b),f(a),0.7],[h(c,b),f(b),0.5],[h(c,b),f(c),0.7],[h(c,b),g(a),0.7],[h(c,b),g(b),0.5],[h(c,b),g(c),0.7],[h(c,c),f(a),0.7],[h(c,c),f(b),0.5],[h(c,c),f(c),0.7],[h(c,c),g(a),0.7],[h(c,c),g(b),0.5],[h(c,c),g(c),0.7]].

test(solve2) :-
    set_prolog_flag(answer_write_options,[max_depth(0)]),
    findall([X1,X2,Deg], solve(p(X1,X2), q(a,d,f(a)), 0.5, Deg), L),
    L == [[h(a,d),f(f(a)),0.8],[h(a,d),f(f(b)),0.5],[h(a,d),f(f(c)),0.7],[h(a,d),f(g(a)),0.7],[h(a,d),f(g(b)),0.5],[h(a,d),f(g(c)),0.7],[h(a,d),g(f(a)),0.7],[h(a,d),g(f(b)),0.5],[h(a,d),g(f(c)),0.7],[h(a,d),g(g(a)),0.7],[h(a,d),g(g(b)),0.5],[h(a,d),g(g(c)),0.7],[h(a,b),f(f(a)),0.7],[h(a,b),f(f(b)),0.5],[h(a,b),f(f(c)),0.7],[h(a,b),f(g(a)),0.7],[h(a,b),f(g(b)),0.5],[h(a,b),f(g(c)),0.7],[h(a,b),g(f(a)),0.7],[h(a,b),g(f(b)),0.5],[h(a,b),g(f(c)),0.7],[h(a,b),g(g(a)),0.7],[h(a,b),g(g(b)),0.5],[h(a,b),g(g(c)),0.7],[h(a,c),f(f(a)),0.8],[h(a,c),f(f(b)),0.5],[h(a,c),f(f(c)),0.7],[h(a,c),f(g(a)),0.7],[h(a,c),f(g(b)),0.5],[h(a,c),f(g(c)),0.7],[h(a,c),g(f(a)),0.7],[h(a,c),g(f(b)),0.5],[h(a,c),g(f(c)),0.7],[h(a,c),g(g(a)),0.7],[h(a,c),g(g(b)),0.5],[h(a,c),g(g(c)),0.7],[h(b,d),f(f(a)),0.5],[h(b,d),f(f(b)),0.5],[h(b,d),f(f(c)),0.5],[h(b,d),f(g(a)),0.5],[h(b,d),f(g(b)),0.5],[h(b,d),f(g(c)),0.5],[h(b,d),g(f(a)),0.5],[h(b,d),g(f(b)),0.5],[h(b,d),g(f(c)),0.5],[h(b,d),g(g(a)),0.5],[h(b,d),g(g(b)),0.5],[h(b,d),g(g(c)),0.5],[h(b,b),f(f(a)),0.5],[h(b,b),f(f(b)),0.5],[h(b,b),f(f(c)),0.5],[h(b,b),f(g(a)),0.5],[h(b,b),f(g(b)),0.5],[h(b,b),f(g(c)),0.5],[h(b,b),g(f(a)),0.5],[h(b,b),g(f(b)),0.5],[h(b,b),g(f(c)),0.5],[h(b,b),g(g(a)),0.5],[h(b,b),g(g(b)),0.5],[h(b,b),g(g(c)),0.5],[h(b,c),f(f(a)),0.5],[h(b,c),f(f(b)),0.5],[h(b,c),f(f(c)),0.5],[h(b,c),f(g(a)),0.5],[h(b,c),f(g(b)),0.5],[h(b,c),f(g(c)),0.5],[h(b,c),g(f(a)),0.5],[h(b,c),g(f(b)),0.5],[h(b,c),g(f(c)),0.5],[h(b,c),g(g(a)),0.5],[h(b,c),g(g(b)),0.5],[h(b,c),g(g(c)),0.5],[h(c,d),f(f(a)),0.7],[h(c,d),f(f(b)),0.5],[h(c,d),f(f(c)),0.7],[h(c,d),f(g(a)),0.7],[h(c,d),f(g(b)),0.5],[h(c,d),f(g(c)),0.7],[h(c,d),g(f(a)),0.7],[h(c,d),g(f(b)),0.5],[h(c,d),g(f(c)),0.7],[h(c,d),g(g(a)),0.7],[h(c,d),g(g(b)),0.5],[h(c,d),g(g(c)),0.7],[h(c,b),f(f(a)),0.7],[h(c,b),f(f(b)),0.5],[h(c,b),f(f(c)),0.7],[h(c,b),f(g(a)),0.7],[h(c,b),f(g(b)),0.5],[h(c,b),f(g(c)),0.7],[h(c,b),g(f(a)),0.7],[h(c,b),g(f(b)),0.5],[h(c,b),g(f(c)),0.7],[h(c,b),g(g(a)),0.7],[h(c,b),g(g(b)),0.5],[h(c,b),g(g(c)),0.7],[h(c,c),f(f(a)),0.7],[h(c,c),f(f(b)),0.5],[h(c,c),f(f(c)),0.7],[h(c,c),f(g(a)),0.7],[h(c,c),f(g(b)),0.5],[h(c,c),f(g(c)),0.7],[h(c,c),g(f(a)),0.7],[h(c,c),g(f(b)),0.5],[h(c,c),g(f(c)),0.7],[h(c,c),g(g(a)),0.7],[h(c,c),g(g(b)),0.5],[h(c,c),g(g(c)),0.7]].

test(solve3) :-
    set_prolog_flag(answer_write_options,[max_depth(0)]),
    findall([X1,X2,Deg], solve(p(X1,f(X2)), q(a,d,g(f(a))), 0.5, Deg), L),
    L == [[h(a,d),g(f(a)),0.8],[h(a,d),g(f(b)),0.5],[h(a,d),g(f(c)),0.7],[h(a,d),g(g(a)),0.7],[h(a,d),g(g(b)),0.5],[h(a,d),g(g(c)),0.7],[h(a,d),f(f(a)),0.7],[h(a,d),f(f(b)),0.5],[h(a,d),f(f(c)),0.7],[h(a,d),f(g(a)),0.7],[h(a,d),f(g(b)),0.5],[h(a,d),f(g(c)),0.7],[h(a,b),g(f(a)),0.7],[h(a,b),g(f(b)),0.5],[h(a,b),g(f(c)),0.7],[h(a,b),g(g(a)),0.7],[h(a,b),g(g(b)),0.5],[h(a,b),g(g(c)),0.7],[h(a,b),f(f(a)),0.7],[h(a,b),f(f(b)),0.5],[h(a,b),f(f(c)),0.7],[h(a,b),f(g(a)),0.7],[h(a,b),f(g(b)),0.5],[h(a,b),f(g(c)),0.7],[h(a,c),g(f(a)),0.8],[h(a,c),g(f(b)),0.5],[h(a,c),g(f(c)),0.7],[h(a,c),g(g(a)),0.7],[h(a,c),g(g(b)),0.5],[h(a,c),g(g(c)),0.7],[h(a,c),f(f(a)),0.7],[h(a,c),f(f(b)),0.5],[h(a,c),f(f(c)),0.7],[h(a,c),f(g(a)),0.7],[h(a,c),f(g(b)),0.5],[h(a,c),f(g(c)),0.7],[h(b,d),g(f(a)),0.5],[h(b,d),g(f(b)),0.5],[h(b,d),g(f(c)),0.5],[h(b,d),g(g(a)),0.5],[h(b,d),g(g(b)),0.5],[h(b,d),g(g(c)),0.5],[h(b,d),f(f(a)),0.5],[h(b,d),f(f(b)),0.5],[h(b,d),f(f(c)),0.5],[h(b,d),f(g(a)),0.5],[h(b,d),f(g(b)),0.5],[h(b,d),f(g(c)),0.5],[h(b,b),g(f(a)),0.5],[h(b,b),g(f(b)),0.5],[h(b,b),g(f(c)),0.5],[h(b,b),g(g(a)),0.5],[h(b,b),g(g(b)),0.5],[h(b,b),g(g(c)),0.5],[h(b,b),f(f(a)),0.5],[h(b,b),f(f(b)),0.5],[h(b,b),f(f(c)),0.5],[h(b,b),f(g(a)),0.5],[h(b,b),f(g(b)),0.5],[h(b,b),f(g(c)),0.5],[h(b,c),g(f(a)),0.5],[h(b,c),g(f(b)),0.5],[h(b,c),g(f(c)),0.5],[h(b,c),g(g(a)),0.5],[h(b,c),g(g(b)),0.5],[h(b,c),g(g(c)),0.5],[h(b,c),f(f(a)),0.5],[h(b,c),f(f(b)),0.5],[h(b,c),f(f(c)),0.5],[h(b,c),f(g(a)),0.5],[h(b,c),f(g(b)),0.5],[h(b,c),f(g(c)),0.5],[h(c,d),g(f(a)),0.7],[h(c,d),g(f(b)),0.5],[h(c,d),g(f(c)),0.7],[h(c,d),g(g(a)),0.7],[h(c,d),g(g(b)),0.5],[h(c,d),g(g(c)),0.7],[h(c,d),f(f(a)),0.7],[h(c,d),f(f(b)),0.5],[h(c,d),f(f(c)),0.7],[h(c,d),f(g(a)),0.7],[h(c,d),f(g(b)),0.5],[h(c,d),f(g(c)),0.7],[h(c,b),g(f(a)),0.7],[h(c,b),g(f(b)),0.5],[h(c,b),g(f(c)),0.7],[h(c,b),g(g(a)),0.7],[h(c,b),g(g(b)),0.5],[h(c,b),g(g(c)),0.7],[h(c,b),f(f(a)),0.7],[h(c,b),f(f(b)),0.5],[h(c,b),f(f(c)),0.7],[h(c,b),f(g(a)),0.7],[h(c,b),f(g(b)),0.5],[h(c,b),f(g(c)),0.7],[h(c,c),g(f(a)),0.7],[h(c,c),g(f(b)),0.5],[h(c,c),g(f(c)),0.7],[h(c,c),g(g(a)),0.7],[h(c,c),g(g(b)),0.5],[h(c,c),g(g(c)),0.7],[h(c,c),f(f(a)),0.7],[h(c,c),f(f(b)),0.5],[h(c,c),f(f(c)),0.7],[h(c,c),f(g(a)),0.7],[h(c,c),f(g(b)),0.5],[h(c,c),f(g(c)),0.7]].

test(best_instances1) :-
    best_instances(p(_,_), q(a,d,f(a)), 0.5, I1),
    I1 == [[p(h(a,d),f(f(a))),0.8],[p(h(a,c),f(f(a))),0.8]],
    best_instances(p(_,f(_)), q(a,d,f(a)), 0.5, I2),
    I2 == [[p(h(a,d),f(f(a))),0.8],[p(h(a,c),f(f(a))),0.8]].

test(all_instances1) :-
    all_instances(p(_,_), q(a,d,a), 0.5, I),
    I == [[p(h(a,b),f(a)),0.7],[p(h(a,b),f(b)),0.5],[p(h(a,b),f(c)),0.7],[p(h(a,b),g(a)),0.7],[p(h(a,b),g(b)),0.5],[p(h(a,b),g(c)),0.7],[p(h(a,c),f(a)),0.8],[p(h(a,c),f(b)),0.5],[p(h(a,c),f(c)),0.7],[p(h(a,c),g(a)),0.7],[p(h(a,c),g(b)),0.5],[p(h(a,c),g(c)),0.7],[p(h(a,d),f(a)),0.8],[p(h(a,d),f(b)),0.5],[p(h(a,d),f(c)),0.7],[p(h(a,d),g(a)),0.7],[p(h(a,d),g(b)),0.5],[p(h(a,d),g(c)),0.7],[p(h(b,b),f(a)),0.5],[p(h(b,b),f(b)),0.5],[p(h(b,b),f(c)),0.5],[p(h(b,b),g(a)),0.5],[p(h(b,b),g(b)),0.5],[p(h(b,b),g(c)),0.5],[p(h(b,c),f(a)),0.5],[p(h(b,c),f(b)),0.5],[p(h(b,c),f(c)),0.5],[p(h(b,c),g(a)),0.5],[p(h(b,c),g(b)),0.5],[p(h(b,c),g(c)),0.5],[p(h(b,d),f(a)),0.5],[p(h(b,d),f(b)),0.5],[p(h(b,d),f(c)),0.5],[p(h(b,d),g(a)),0.5],[p(h(b,d),g(b)),0.5],[p(h(b,d),g(c)),0.5],[p(h(c,b),f(a)),0.7],[p(h(c,b),f(b)),0.5],[p(h(c,b),f(c)),0.7],[p(h(c,b),g(a)),0.7],[p(h(c,b),g(b)),0.5],[p(h(c,b),g(c)),0.7],[p(h(c,c),f(a)),0.7],[p(h(c,c),f(b)),0.5],[p(h(c,c),f(c)),0.7],[p(h(c,c),g(a)),0.7],[p(h(c,c),g(b)),0.5],[p(h(c,c),g(c)),0.7],[p(h(c,d),f(a)),0.7],[p(h(c,d),f(b)),0.5],[p(h(c,d),f(c)),0.7],[p(h(c,d),g(a)),0.7],[p(h(c,d),g(b)),0.5],[p(h(c,d),g(c)),0.7]].

:- end_tests(rel4).
